#ifndef IU9_CA_WEB_CHAT_LIB_BACKEND_LOGIC_SERVER_DATA_INTERACT_H
#define IU9_CA_WEB_CHAT_LIB_BACKEND_LOGIC_SERVER_DATA_INTERACT_H

/* This folder covers all code that helps to interact with database and templater,
*  or dictates the logic of how this web chat functions */

#include "../sqlite3_wrapper.h"
#include <jsonincpp/string_representation.h>

namespace iu9cawebchat {
    constexpr int64_t user_chat_role_admin = 1;
    constexpr int64_t user_chat_role_regular = 2;
    constexpr int64_t user_chat_role_read_only = 3;
    constexpr int64_t user_chat_role_deleted = 4;

    const char* stringify_user_chat_role(int64_t role);

    int64_t find_user_by_credentials (SqliteConnection& conn, const std::string& nickname, const std::string& password);
    std::string find_user_name (SqliteConnection& conn, int64_t uid);

    struct RowUser_Content {
        std::string nickname;
        std::string name;
    };

    struct RowChat_Content {
        std::string nickname;
        std::string name;
        int64_t lastMsgId;  // Negative if it does not exist
    };

    RowUser_Content lookup_user_content(SqliteConnection& conn, int64_t uid);
    RowChat_Content lookup_chat_content(SqliteConnection& conn, int64_t chatId);

    struct RowMessage_Content {
        bool isSystem;
        std::string text;
        int64_t senderUserId;
        int64_t previous = -1;
    };

    RowMessage_Content lookup_message_content(SqliteConnection& conn, int64_t chatId, int64_t msgId);

    int64_t get_role_of_user_in_chat(SqliteConnection& conn, int64_t userId, int64_t chatId);

    /* ============================= API ====================================*/
    json::JSON internalapi_pollEvents(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);
    json::JSON internalapi_getChatList(SqliteConnection& conn, int64_t uid);
    json::JSON internalapi_getChatInfo(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);
    json::JSON internalapi_getChatMemberList(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);
    json::JSON internalapi_getUserInfo(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);
    json::JSON internalapi_getMessageInfo(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);
    // todo: complete the list
}

#endif
